let step;
let length = [3, 3, 4, 4, 5, 5];
let input = [];

let canPress;

let range = ["0 to 7", "0 to 9", "0 to 9, A to F", "A to Z"];

let invalid = false;
let valid = false;

let tmr;
let stop;

function play(currLvl) {
    clear();

    step = 0;

    cvs.style.cursor = "default";

    theme1.pause();
    theme2.play();

    cvs.onmousemove = null
    cvs.onmousedown = null;

    let sec = 0;

    function update() {
        clear();

        if (sec === 6 * 60) stop();

        ctx.drawImage(imgClock, w/3 - 24, 48 - 12, 24, 24);
        ctx.drawImage(imgFlag, w/3 - 24, 80 - 12, 24, 24);
        ctx.font = "24px serif";
        ctx.fillText(`${Math.floor(sec / 60)}:${(sec % 60).toString().padStart(2, "0")}/6:00`, w/3 + 8, 48);
        ctx.fillText(`${step}/6`, w/3 + 8, 80);

        for (let i = 0; i < length[step]; i++) {
            ctx.font = "48px serif";
            text(input[i] || "-", (i+1) * w / (length[step]+1), h/3 + 16);
        }

        ctx.save();
        ctx.beginPath();
        ctx.arc(w/3, h * 0.6, 16, 0, 7);
        ctx.fillStyle = invalid ? "#F00" : "#700";
        ctx.fill();
        ctx.closePath();

        ctx.beginPath();
        ctx.arc(2*w/3, h * 0.6, 16, 0, 7);
        ctx.fillStyle = valid ? "#0F0" : "#070";
        ctx.fill();
        ctx.closePath();
        ctx.restore();

        ctx.save();
        ctx.font = "20px serif";
        text(`Find the ${length[step]}-character code!`, w/2, h * 0.8);
        text(`(character set: ${range[currLvl - 1]})`, w/2, h * 0.8 + 32);
        ctx.restore();
    }

    update();
    tmr = setInterval(function eachSecond() {
        sec++;
        update();
    }, 1000);

    document.body.onkeydown = function keydown(evt) {
        if (!canPress) return;

        canPress = false;

        let key = evt.key.toUpperCase();

        if (/^[0-9A-Z]$/.test(key)) {
            if (isValid([...input, key])) {
                valid = true;
                invalid = false;
                input.push(key);
                update();

                if (input.length === code.length) {
                    step++;
                    input = [];
                    update();

                    nextSound.pause();
                    nextSound.currentTime = 0;
                    nextSound.play();

                    if (step == 6) {
                        stop();
                    }

                    isValid = generate();
                } else {
                    validSound.play();
                }
            } else {
                valid = false;
                invalid = true;
                input = [];
                update();
            }
        }
    };

    document.body.onkeyup = function keyup(evt) {
        canPress = true;
        valid = false;
        invalid = false;
        update();
    }

    function generate() {
        let pool = [
            "01234567",
            "0123456789",
            "0123456789ABCDEF",
            "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        ][currLvl - 1];

        code = [];
        for (let i = 0; i < length[step]; i++) {
            code.push(pool[Math.floor(Math.random() * pool.length)]);
        }

        return function isValid(inp) {
            for (let i = 0; i < inp.length; i++) {
                if (inp[i] !== code[i]) return false;
            }

            return true;
        }
    }

    let code;
    let isValid = generate();

    stop = function stopGame() {
        clearInterval(tmr);
        clear();

        if (currLvl === 1) level1 = Math.max(step, level1);
        if (currLvl === 2) level2 = Math.max(step, level2);
        if (currLvl === 3) level3 = Math.max(step, level3);
        if (currLvl === 4) level4 = Math.max(step, level4);

        document.body.onkeydown = null;

        setTimeout(function reset() {
            theme2.pause();
            theme2.currentTime = 0;

            menu();
        }, 1000);
    }
}
